@@ -30,7 +30,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#include <gnome.h>
+#include <gtk/gtk.h>
 #include "gok-spy-priv.h"
 #include "gok-output.h"
 #include "gok-word-complete.h"
