@@ -1,7 +1,7 @@
 /* gok-page-accessmethod.c
 *
-* Copyright 2002 Sun Microsystems, Inc.,
-* Copyright 2002 University Of Toronto
+* Copyright 2002-2009 Sun Microsystems, Inc.,
+* Copyright 2002-2009 University Of Toronto
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
@@ -44,11 +44,11 @@ typedef struct GokFilledCombo{
 } GokFilledCombo;
 
 /* settings for the access method */
-static gchar m_NameAccessMethod [MAX_ACCESS_METHOD_NAME + 1];
+static gchar* m_NameAccessMethod = NULL;
 static GokAccessMethod* m_pAccessMethod;
 
 /* backup of the settings (in case the "cancel" button is clicked) */
-static gchar m_NameAccessMethodBackup [MAX_ACCESS_METHOD_NAME + 1];
+static gchar* m_NameAccessMethodBackup = NULL;
 
 /* this flag will be TRUE when we are changing the text in the combo entry */
 static gboolean m_bUsChangingName;
@@ -57,8 +57,8 @@ static gboolean m_bUsChangingName;
 static GokFilledCombo* m_pGokFilledComboFirst;
 
 /**
-* gok-settings-page-accessmethod-initialize
-* @pWindowSettings: Pointer to the settings dialog window.
+* gok_page_accessmethod_initialize
+* @xml: Pointer to the Glade UI description of the settings dialog.
 *
 * Initializes this page of the gok settings dialog. This must be called
 * prior to any calls on this page.
@@ -75,7 +75,7 @@ gboolean gok_page_accessmethod_initialize (GladeXML* xml)
 	g_assert (xml != NULL);
 	
 	/* initialize this data */
-	strcpy (m_NameAccessMethod, gok_data_get_name_accessmethod());
+	m_NameAccessMethod = g_strdup (gok_data_get_name_accessmethod());
 	m_bUsChangingName = FALSE;
 	m_pAccessMethod = NULL;
 	m_pGokFilledComboFirst = NULL;
@@ -141,6 +141,8 @@ void gok_page_accessmethod_close()
 		}
 		g_free (pGokFilledComboTemp);
 	}
+	g_free (m_NameAccessMethod);
+	g_free (m_NameAccessMethodBackup);
 }
 
 /**
@@ -323,7 +325,7 @@ gboolean gok_page_accessmethod_apply_controls (GokControl* pControl, gchar* Name
 						g_assert (pRadioButton != NULL);
 						if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(pRadioButton)) == TRUE)
 						{
-							pRadioControl = (GokControl*)gtk_object_get_data (GTK_OBJECT(pRadioButton), "control");
+							pRadioControl = (GokControl*) g_object_get_data (G_OBJECT (pRadioButton), "control");
 							g_assert (pRadioControl != NULL);
 							if (gok_data_set_setting (NameAccessMethod, pControl->Name, pRadioControl->Value, NULL) == TRUE)
 							{
@@ -382,7 +384,8 @@ gboolean gok_page_accessmethod_revert ()
 	if (strcmp (m_NameAccessMethod, m_NameAccessMethodBackup) != 0)
 	{
 		bDataChanged = TRUE;
-		strcpy (m_NameAccessMethod, m_NameAccessMethodBackup);
+		g_free (m_NameAccessMethod);
+		m_NameAccessMethod = g_strdup (m_NameAccessMethodBackup);
 		gok_data_set_name_accessmethod (m_NameAccessMethod);
 		gok_page_accessmethod_change_controls (m_NameAccessMethod);
 		gok_scanner_change_method (m_NameAccessMethod);
@@ -408,7 +411,8 @@ gboolean gok_page_accessmethod_revert ()
 **/
 void gok_page_accessmethod_backup ()
 {
-	strcpy (m_NameAccessMethodBackup, m_NameAccessMethod);
+	g_free (m_NameAccessMethodBackup);
+	m_NameAccessMethodBackup = g_strdup (m_NameAccessMethod);
 }
 
 /**
@@ -479,8 +483,7 @@ void gok_page_accessmethod_update_controls (gchar* NameAccessMethod, GokControl*
 							{
 								g_free (pControl->String);
 							}
-							pControl->String = (gchar*)g_malloc (strlen (settingString) + 1);
-							strcpy (pControl->String, settingString);
+							pControl->String = g_strdup (settingString);
 							
 							if (pControl->pWidget != NULL)
 							{
@@ -546,19 +549,20 @@ void gok_page_accessmethod_update_controls (gchar* NameAccessMethod, GokControl*
 * gok_page_accessmethod_draw_controls
 * @NameAccessMethod: Name of the access methods that needs the controls.
 * @pControlParent: Pointer to the GOK control that will contain the new controls.
-* @pContainter: Pointer to the container widget for the controls.
 * @pControl: Pointer to the GokControl that you want drawn.
+* @pSizeGroup: Pointer to the GtkSizeGroup the control belongs to.
 * @bShow: If TRUE then the controls will be show, if FALSE they will be hidden.
 * 
 * Draws the controls for the GokControl.
 **/
-void gok_page_accessmethod_draw_controls (gchar* NameAccessMethod, GokControl* pControlParent, GokControl* pControl, gboolean bShow)
+void gok_page_accessmethod_draw_controls (gchar* NameAccessMethod, GokControl* pControlParent, GokControl* pControl, GtkSizeGroup* pSizeGroup, gboolean bShow)
 {
 	GtkWidget* pWidget;
 	GtkWidget* pEntry;
 	static GSList* ListRadioButtons;
 	GtkObject* spinRateAdjustment;
 	gchar* pControlName;
+	gchar* pFrameHeader;
 	int settingInt;
 	gchar* settingString;
 	GokFilledCombo* pFilledComboNew;
@@ -576,22 +580,31 @@ void gok_page_accessmethod_draw_controls (gchar* NameAccessMethod, GokControl* p
 			}
 			else /* create the control */
 			{
+				/* create a new size group */
+				if (pSizeGroup == NULL)
+					pSizeGroup = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+
 				switch (pControl->Type)
 				{
 					case CONTROL_TYPE_LABEL:
-						pWidget = gtk_label_new (_(pControl->String));
-						gtk_widget_ref (pWidget);
-						gtk_object_set_data_full (GTK_OBJECT (gok_settingsdialog_get_window()), "GOKlabel", pWidget,
-						                            (GtkDestroyNotify) gtk_widget_unref);
+						pWidget = gtk_label_new (pControl->String);
+						g_object_set (pWidget, "xalign", 0., NULL);
+						g_object_ref (pWidget);
+						g_object_set_data_full (G_OBJECT (gok_settingsdialog_get_window ()),
+									"GOKlabel", pWidget,
+									(GDestroyNotify) g_object_unref);
 						gtk_widget_show (pWidget);
 						gtk_box_pack_start (GTK_BOX (pControlParent->pWidget), pWidget, FALSE, FALSE, 0);
+						gtk_size_group_add_widget (pSizeGroup, pWidget);
 						break;
 						
 					case CONTROL_TYPE_HBOX:
 						pWidget = gtk_hbox_new (FALSE, pControl->Spacing);
-						gtk_widget_ref (pWidget);
-						gtk_object_set_data_full (GTK_OBJECT (gok_settingsdialog_get_window()), "GOKhbox", pWidget,
-						                            (GtkDestroyNotify) gtk_widget_unref);
+						g_object_ref (pWidget);
+						g_object_set_data_full (G_OBJECT (gok_settingsdialog_get_window ()),
+									"GOKhbox", pWidget,
+									(GDestroyNotify) g_object_unref);
+						gtk_container_set_border_width (GTK_CONTAINER (pWidget), pControl->Border);
 						gtk_widget_show (pWidget);
 						
 						if (pControlParent->Type == CONTROL_TYPE_FRAME)
@@ -601,15 +614,16 @@ void gok_page_accessmethod_draw_controls (gchar* NameAccessMethod, GokControl* p
 						else
 						{
 							gtk_box_pack_start (GTK_BOX (pControlParent->pWidget), pWidget, TRUE, TRUE, 0);
-							gtk_container_set_border_width (GTK_CONTAINER (pControlParent->pWidget), pControl->Border);
 						}
 						break;
 						
 					case CONTROL_TYPE_VBOX:
 						pWidget = gtk_vbox_new (FALSE, pControl->Spacing);
-						gtk_widget_ref (pWidget);
-						gtk_object_set_data_full (GTK_OBJECT (gok_settingsdialog_get_window()), "GOKvbox", pWidget,
-						                            (GtkDestroyNotify) gtk_widget_unref);
+						g_object_ref (pWidget);
+						g_object_set_data_full (G_OBJECT (gok_settingsdialog_get_window ()),
+									"GOKvbox", pWidget,
+									(GDestroyNotify) g_object_unref);
+						gtk_container_set_border_width (GTK_CONTAINER (pWidget), pControl->Border);
 						gtk_widget_show (pWidget);
 						
 						if (pControlParent->Type == CONTROL_TYPE_FRAME)
@@ -619,16 +633,16 @@ void gok_page_accessmethod_draw_controls (gchar* NameAccessMethod, GokControl* p
 						else
 						{
 							gtk_box_pack_start (GTK_BOX (pControlParent->pWidget), pWidget, TRUE, TRUE, 0);
-							gtk_container_set_border_width (GTK_CONTAINER (pControlParent->pWidget), pControl->Border);
 						}
 						break;
 						
 					case CONTROL_TYPE_COMBOBOX:
 						pControlName = (pControl->Name != NULL) ? pControl->Name : "combobox";
 						pWidget = gtk_combo_new ();
-						gtk_widget_ref (pWidget);
-						gtk_object_set_data_full (GTK_OBJECT (gok_settingsdialog_get_window()), pControlName, pWidget,
-						                            (GtkDestroyNotify) gtk_widget_unref);
+						g_object_ref (pWidget);
+						g_object_set_data_full (G_OBJECT (gok_settingsdialog_get_window ()),
+									pControlName, pWidget,
+									(GDestroyNotify) g_object_unref);
 						atko = gtk_widget_get_accessible (pWidget);
 						atk_object_set_name (atko, pControl->String ? pControl->String : "");
 						gtk_widget_show (pWidget);
@@ -636,7 +650,7 @@ void gok_page_accessmethod_draw_controls (gchar* NameAccessMethod, GokControl* p
 
 						/* make the combo's entry field NOT editable */
 						pEntry = GTK_COMBO (pWidget)->entry;
-						gtk_entry_set_editable (GTK_ENTRY (pEntry), FALSE);
+						gtk_editable_set_editable (GTK_EDITABLE (pEntry), FALSE);
 						
 						/* create a new object to hold info about the filled combo */
 						/* the combo gets filled in gok_page_accessmethod_fill_combos */
@@ -644,10 +658,8 @@ void gok_page_accessmethod_draw_controls (gchar* NameAccessMethod, GokControl* p
 						pFilledComboNew->Fillwith = pControl->Fillwith;
 						pFilledComboNew->Qualifier = pControl->Qualifier;
 						pFilledComboNew->pCombo = pWidget;
-						pFilledComboNew->pNameAccessMethod = (gchar*)g_malloc (strlen (NameAccessMethod) + 1);
-						strcpy (pFilledComboNew->pNameAccessMethod, NameAccessMethod);
-						pFilledComboNew->pNameControl = (gchar*)g_malloc (strlen (pControlName) + 1);
-						strcpy (pFilledComboNew->pNameControl, pControlName);
+						pFilledComboNew->pNameAccessMethod = g_strdup (NameAccessMethod);
+						pFilledComboNew->pNameControl = g_strdup (pControlName);
 						pFilledComboNew->pGokFilledComboNext = NULL;
 						pFilledComboNew->bFilled = FALSE;
 						
@@ -669,28 +681,35 @@ void gok_page_accessmethod_draw_controls (gchar* NameAccessMethod, GokControl* p
 						
 					case CONTROL_TYPE_SEPERATOR:
 						pWidget = gtk_hseparator_new ();
-						gtk_widget_ref (pWidget);
-						gtk_object_set_data_full (GTK_OBJECT (gok_settingsdialog_get_window()), "GOKseparator", pWidget,
-						                            (GtkDestroyNotify) gtk_widget_unref);
+						g_object_ref (pWidget);
+						g_object_set_data_full (G_OBJECT (gok_settingsdialog_get_window ()),
+									"GOKseparator", pWidget,
+									(GDestroyNotify) g_object_unref);
 						gtk_widget_show (pWidget);
 						gtk_box_pack_start (GTK_BOX (pControlParent->pWidget), pWidget, TRUE, TRUE, 0);
 						break;
 						
 					case CONTROL_TYPE_FRAME:
-						pWidget = gtk_frame_new (_(pControl->String));
-						gtk_widget_ref (pWidget);
-						gtk_object_set_data_full (GTK_OBJECT (gok_settingsdialog_get_window()), "GOKframe", pWidget,
-						                            (GtkDestroyNotify) gtk_widget_unref);
+						pFrameHeader = g_markup_printf_escaped ("<b>%s</b>", pControl->String);
+						pWidget = gtk_frame_new (pFrameHeader);
+						g_free (pFrameHeader);
+						g_object_ref (pWidget);
+						g_object_set_data_full (G_OBJECT (gok_settingsdialog_get_window ()),
+									"GOKframe", pWidget,
+									(GDestroyNotify) g_object_unref);
+						gtk_label_set_use_markup (GTK_LABEL (gtk_frame_get_label_widget (GTK_FRAME (pWidget))), TRUE);
+						gtk_frame_set_shadow_type (GTK_FRAME (pWidget), GTK_SHADOW_NONE);
 						gtk_widget_show (pWidget);
 						gtk_box_pack_start (GTK_BOX (pControlParent->pWidget), pWidget, FALSE, FALSE, 0);
 						break;
 						
 					case CONTROL_TYPE_BUTTON:
 						pControlName = (pControl->Name != NULL) ? pControl->Name : "button";
-						pWidget = gtk_button_new_with_label (_(pControl->String));
-						gtk_widget_ref (pWidget);
-						gtk_object_set_data_full (GTK_OBJECT (gok_settingsdialog_get_window()), pControlName, pWidget,
-						                            (GtkDestroyNotify) gtk_widget_unref);
+						pWidget = gtk_button_new_with_label (pControl->String);
+						g_object_ref (pWidget);
+						g_object_set_data_full (G_OBJECT (gok_settingsdialog_get_window ()),
+									pControlName, pWidget,
+									(GDestroyNotify) g_object_unref);
 						gtk_widget_show (pWidget);
 						gtk_box_pack_start (GTK_BOX (pControlParent->pWidget), pWidget, FALSE, FALSE, 0);
 						
@@ -698,10 +717,11 @@ void gok_page_accessmethod_draw_controls (gchar* NameAccessMethod, GokControl* p
 						break;
 						
 					case CONTROL_TYPE_CHECKBUTTON:
-						pWidget = gtk_check_button_new_with_label (_(pControl->String));
-						gtk_widget_ref (pWidget);
-						gtk_object_set_data_full (GTK_OBJECT (gok_settingsdialog_get_window()), "GOKcheckbutton", pWidget,
-						                            (GtkDestroyNotify) gtk_widget_unref);
+						pWidget = gtk_check_button_new_with_label (pControl->String);
+						g_object_ref (pWidget);
+						g_object_set_data_full (G_OBJECT (gok_settingsdialog_get_window ()),
+									"GOKcheckbutton", pWidget,
+									(GDestroyNotify) g_object_unref);
 						gtk_widget_show (pWidget);
 						gtk_box_pack_start (GTK_BOX (pControlParent->pWidget), pWidget, FALSE, FALSE, 0);
 
@@ -717,8 +737,8 @@ void gok_page_accessmethod_draw_controls (gchar* NameAccessMethod, GokControl* p
 						/* connect this signal so we get notified when the control changes state */
 						/* note: This is only done for checkboxes. If you have associated controls
 						   for other controls then add a handler for them. */
-						gtk_signal_connect (GTK_OBJECT (pWidget), "toggled",
-						GTK_SIGNAL_FUNC (gok_page_accessmethod_checkbox_changed),NULL);
+						g_signal_connect (pWidget, "toggled",
+								  G_CALLBACK (gok_page_accessmethod_checkbox_changed), NULL);
 						break;
 						
 					case CONTROL_TYPE_RADIOBUTTON:
@@ -727,15 +747,16 @@ void gok_page_accessmethod_draw_controls (gchar* NameAccessMethod, GokControl* p
 							ListRadioButtons = NULL;
 						}
 
-						pWidget = gtk_radio_button_new_with_label (ListRadioButtons, _(pControl->String));
-						ListRadioButtons = gtk_radio_button_group (GTK_RADIO_BUTTON (pWidget));
-						gtk_widget_ref (pWidget);
-						gtk_object_set_data_full (GTK_OBJECT (gok_settingsdialog_get_window()), "GOKradiobutton", pWidget,
-						                            (GtkDestroyNotify) gtk_widget_unref);
+						pWidget = gtk_radio_button_new_with_label (ListRadioButtons, pControl->String);
+						ListRadioButtons = gtk_radio_button_get_group (GTK_RADIO_BUTTON (pWidget));
+						g_object_ref (pWidget);
+						g_object_set_data_full (G_OBJECT (gok_settingsdialog_get_window ()),
+									"GOKradiobutton", pWidget,
+									(GDestroyNotify) g_object_unref);
 						gtk_widget_show (pWidget);
 						gtk_box_pack_start (GTK_BOX (pControlParent->pWidget), pWidget, FALSE, FALSE, 0);
 						
-						gtk_object_set_data(GTK_OBJECT (pWidget), "control", pControl);
+						g_object_set_data (G_OBJECT (pWidget), "control", pControl);
 
 						if ((pControl->Name != NULL) &&
 							(gok_data_get_setting (NameAccessMethod, pControl->Name, &settingInt, &settingString) == TRUE))
@@ -749,16 +770,17 @@ void gok_page_accessmethod_draw_controls (gchar* NameAccessMethod, GokControl* p
 						
 					case CONTROL_TYPE_SPINBUTTON:
 						spinRateAdjustment = gtk_adjustment_new (pControl->Value, 
-																pControl->Min, 
-																pControl->Max, 
-																pControl->StepIncrement,
-																pControl->PageIncrement,
-																pControl->PageSize);
+											 pControl->Min, 
+											 pControl->Max, 
+											 pControl->StepIncrement,
+											 pControl->PageIncrement,
+											 pControl->PageSize);
 		
 						pWidget = gtk_spin_button_new (GTK_ADJUSTMENT (spinRateAdjustment), 1, 0);
-						gtk_widget_ref (pWidget);
-						gtk_object_set_data_full (GTK_OBJECT (gok_settingsdialog_get_window()), "GOKspinbutton", pWidget,
-						                            (GtkDestroyNotify) gtk_widget_unref);
+						g_object_ref (pWidget);
+						g_object_set_data_full (G_OBJECT (gok_settingsdialog_get_window ()),
+									"GOKspinbutton", pWidget,
+									(GDestroyNotify) g_object_unref);
 						gtk_widget_show (pWidget);
 						gtk_box_pack_start (GTK_BOX (pControlParent->pWidget), pWidget, FALSE, FALSE, 0);
 
@@ -787,7 +809,7 @@ void gok_page_accessmethod_draw_controls (gchar* NameAccessMethod, GokControl* p
 		/* create any child controls */
 		if (pControl->pControlChild != NULL)
 		{
-			gok_page_accessmethod_draw_controls (NameAccessMethod, pControl, pControl->pControlChild, bShow);
+			gok_page_accessmethod_draw_controls (NameAccessMethod, pControl, pControl->pControlChild, pSizeGroup, bShow);
 		}
 
 		pControl = pControl->pControlNext;
@@ -808,8 +830,8 @@ void gok_page_accessmethod_checkbox_changed (GtkButton* pButton, gpointer data)
 	GokControl* pControl;
 	GokControl* pControlAssociated;
 	gchar* token;
-	gchar seperators[] = "+";
-	gchar buffer[151];
+	gchar **tokens;
+	gint i;
 
 	/* find the GokControl that corresponds to the checkbox */
 	pAccessMethod = gok_scanner_get_first_access_method();
@@ -835,10 +857,10 @@ void gok_page_accessmethod_checkbox_changed (GtkButton* pButton, gpointer data)
 	if (pControl->NameAssociatedControl != NULL)
 	{
 		/* there may be several associated controls so parse the string */
-		strncpy (buffer, pControl->NameAssociatedControl, 150);
-		token = strtok (buffer, seperators);
-		while (token != NULL)
+		tokens = g_strsplit (pControl->NameAssociatedControl, "+", -1);
+		for (i = 0; tokens [i] != NULL; i++)
 		{
+			token = tokens [i];
 			/* find the GokControl for the associated control */
 			pControlAssociated = gok_control_find_by_name (token, pAccessMethod->pControlOptions);
 			if (pControlAssociated == NULL)
@@ -871,9 +893,8 @@ void gok_page_accessmethod_checkbox_changed (GtkButton* pButton, gpointer data)
 				}
 			}
 
-			/* get the next associated control */
-			token = strtok (NULL, seperators);
 		}
+		g_strfreev (tokens);
 	}
 }
 
@@ -907,13 +928,14 @@ void gok_page_accessmethod_change_controls (gchar* pNameAccessMethod)
 			/* hide the controls from the previous access method */
 			if (m_pAccessMethod != NULL)
 			{
-				gok_page_accessmethod_draw_controls (pNameAccessMethod, &ControlParent, m_pAccessMethod->pControlOperation, FALSE);
-				gok_page_accessmethod_draw_controls (pNameAccessMethod, &ControlParent, m_pAccessMethod->pControlFeedback, FALSE);
-				gok_page_accessmethod_draw_controls (pNameAccessMethod, &ControlParent, m_pAccessMethod->pControlOptions, FALSE);
+				gok_page_accessmethod_draw_controls (pNameAccessMethod, &ControlParent, m_pAccessMethod->pControlOperation, NULL, FALSE);
+				gok_page_accessmethod_draw_controls (pNameAccessMethod, &ControlParent, m_pAccessMethod->pControlFeedback, NULL, FALSE);
+				gok_page_accessmethod_draw_controls (pNameAccessMethod, &ControlParent, m_pAccessMethod->pControlOptions, NULL, FALSE);
 			}
 
 			/* store the new access method */
-			strcpy (m_NameAccessMethod, pNameAccessMethod);
+			g_free (m_NameAccessMethod);
+			m_NameAccessMethod = g_strdup (pNameAccessMethod);
 			m_pAccessMethod = pAccessMethod;
 			
 			/* display the name in the combo box */
@@ -929,9 +951,9 @@ void gok_page_accessmethod_change_controls (gchar* pNameAccessMethod)
 			gtk_label_set_text (GTK_LABEL (pLabelDescription), pAccessMethod->Description);
 				
 			/* display the controls for the selected access method */
-			gok_page_accessmethod_draw_controls (pNameAccessMethod, &ControlParent, pAccessMethod->pControlOperation, TRUE);
-			gok_page_accessmethod_draw_controls (pNameAccessMethod, &ControlParent, pAccessMethod->pControlFeedback, TRUE);
-			gok_page_accessmethod_draw_controls (pNameAccessMethod, &ControlParent, pAccessMethod->pControlOptions, TRUE);
+			gok_page_accessmethod_draw_controls (pNameAccessMethod, &ControlParent, pAccessMethod->pControlOperation, NULL, TRUE);
+			gok_page_accessmethod_draw_controls (pNameAccessMethod, &ControlParent, pAccessMethod->pControlFeedback, NULL, TRUE);
+			gok_page_accessmethod_draw_controls (pNameAccessMethod, &ControlParent, pAccessMethod->pControlOptions, NULL, TRUE);
 
 			gok_page_accessmethod_update_associated (pAccessMethod);
 			return;
